
//
// This source file is part of appleseed.
// Visit http://appleseedhq.net/ for additional information and resources.
//
// This software is released under the MIT license.
//
// Copyright (c) 2010-2013 Francois Beaune, Jupiter Jazz Limited
// Copyright (c) 2014 Francois Beaune, The appleseedhq Organization
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//

// appleseed.foundation headers.
#include "foundation/image/color.h"
#include "foundation/image/colorspace.h"
#include "foundation/image/spectrum.h"
#include "foundation/math/rng.h"
#include "foundation/utility/benchmark.h"

// Standard headers.
#include <cstddef>

using namespace foundation;

BENCHMARK_SUITE(Foundation_Image_ColorSpace)
{
    struct RGBFixture
    {
        Color3f m_input;
        Color3f m_output;

        RGBFixture()
          : m_input(0.5f, 0.7f, 0.2f)
        {
        }
    };

    BENCHMARK_CASE_F(LinearRGBTosRGBConversion, RGBFixture)
    {
        m_output = linear_rgb_to_srgb(m_input);
    }

    BENCHMARK_CASE_F(FastLinearRGBTosRGBConversion, RGBFixture)
    {
        m_output = fast_linear_rgb_to_srgb(m_input);
    }

    struct SpectrumFixture
    {
        typedef RegularSpectrum<float, 31> SpectrumType;

        const LightingConditions    m_lighting_conditions;
        SpectrumType                m_input;
        Color3f                     m_output;

        SpectrumFixture()
          : m_lighting_conditions(IlluminantCIED65, XYZCMFCIE196410Deg)
        {
            MersenneTwister rng;

            for (size_t w = 0; w < SpectrumType::Samples; ++w)
                m_input[w] = rand_float1(rng);
        }
    };

    BENCHMARK_CASE_F(SpectrumToCIEXYZ, SpectrumFixture)
    {
        m_output = spectrum_to_ciexyz<float>(m_lighting_conditions, m_input);
    }
}
